module MPYsm ( RESET, CLK, START, MAX, SM );
`include "MPYdefs.v"
  input RESET, CLK, START, MAX;
  output [1:0] SM;
  reg [1:0] Sreg, Snext;

  always @ (posedge CLK) // state memory (w/ sync. reset)
    if (RESET) Sreg <= IDLE;
    else Sreg <= Snext;

  always @ (START or MAX or Sreg) // next-state logic
    case (Sreg)
      IDLE : if (START)             Snext <= INIT;
             else                   Snext <= IDLE;
      INIT :                        Snext <= RUN;
      RUN  : if (MAX && ~START)     Snext <= IDLE;
             else if (MAX && START) Snext <= WAIT;
             else                   Snext <= RUN;
      WAIT : if (~START)            Snext <= IDLE;
             else                   Snext <= WAIT;
      default :                     Snext <= IDLE;
    endcase

  assign SM = Sreg;  // Copy state to module output
endmodule