module Vrggame ( CLOCK, RESET, G1, G2, G3, G4, L1, L2, L3, L4, ERR );
  input CLOCK, RESET, G1, G2, G3, G4;
  output L1, L2, L3, L4, ERR;
  reg L1, L2, L3, L4, ERR;
  reg [2:0] Sreg, Snext;    // State register and next state
  parameter S1   = 3'b001,  // State encodings for 4 running states,
            S2   = 3'b010,
            S3   = 3'b011,
            S4   = 3'b100,
            SOK  = 3'b101,  // OK state, and
            SERR = 3'b110;  // error state

  always @ (posedge CLOCK)	// Create state memory with sync reset
    if (RESET) Sreg <= SOK; else Sreg <= Snext;

  always @ (G1 or G2 or G3 or G4 or Sreg)   // Next-state logic
    case (Sreg)
      S1  : if (G2 || G3 || G4) Snext = SERR;
            else if (G1)        Snext = SOK;
            else                Snext = S2;
      S2  : if (G1 || G3 || G4) Snext = SERR;
            else if (G2)        Snext = SOK;
            else                Snext = S3;
      S3  : if (G1 || G2 || G4) Snext = SERR;
            else if (G3)        Snext = SOK;
            else                Snext = S4;
      S4  : if (G1 || G2 || G3) Snext = SERR;
            else if (G4)        Snext = SOK;
            else                Snext = S1;
      SOK : if (~G1 & ~G2 & ~G3 & ~G4) Snext = S1; else Snext = SOK;
      SERR: if (~G1 & ~G2 & ~G3 & ~G4) Snext = S1; else Snext = SERR;
      default : Snext = S1;
    endcase

  always @ (Sreg) begin      // Output logic
    L1  = (Sreg == S1);
    L2  = (Sreg == S2);
    L3  = (Sreg == S3);
    L4  = (Sreg == S4);
    ERR = (Sreg == SERR);
  end
endmodule