module Vrggameoc ( CLOCK, RESET, G1, G2, G3, G4, L1, L2, L3, L4, ERR );
  input CLOCK, RESET, G1, G2, G3, G4;
  output L1, L2, L3, L4, ERR;
  reg L1, L2, L3, L4, ERR;
  reg [1:5] Sreg, Snext;      // State register and next state
  // The five state bits correspond to outputs L1, L2, L3, L4, ERR
  parameter S1   = 5'b10000,  // Output coded state assignment
            S2   = 5'b01000,  //  for 4 running states,
            S3   = 5'b00100,
            S4   = 5'b00010,
            SOK  = 5'b00000,  // OK state, and
            SERR = 5'b00001;  // error state

  always @ (posedge CLOCK)	// Create state memory with sync reset
    if (RESET) Sreg <= SOK; else Sreg <= Snext;

  always @ (G1 or G2 or G3 or G4 or Sreg)   // Next-state logic
    case (Sreg)
      S1  : if (G2 || G3 || G4) Snext = SERR;
            else if (G1)        Snext = SOK;
            else                Snext = S2;
      S2  : if (G1 || G3 || G4) Snext = SERR;
            else if (G2)        Snext = SOK;
            else                Snext = S3;
      S3  : if (G1 || G2 || G4) Snext = SERR;
            else if (G3)        Snext = SOK;
            else                Snext = S4;
      S4  : if (G1 || G2 || G3) Snext = SERR;
            else if (G4)        Snext = SOK;
            else                Snext = S1;
      SOK : if (~G1 & ~G2 & ~G3 & ~G4) Snext = S1; else Snext = SOK;
      SERR: if (~G1 & ~G2 & ~G3 & ~G4) Snext = S1; else Snext = SERR;
      default : Snext = S1;
    endcase

  always @ (Sreg) begin   // Output logic
    L1  = Sreg[1];
    L2  = Sreg[2];
    L3  = Sreg[3];
    L4  = Sreg[4];
    ERR = Sreg[5];
  end
endmodule