module Vrsvale ( CLOCK, RESET, NSCAR, EWCAR, TMSHORT, TMLONG,
                 OVERRIDE, FLASHCLK, NSRED, NSYELLOW, NSGREEN,
                 EWRED, EWYELLOW, EWGREEN, TMRESET );
  input CLOCK, RESET, NSCAR, EWCAR, TMSHORT, TMLONG, OVERRIDE, FLASHCLK;
  output NSRED, NSYELLOW, NSGREEN, EWRED, EWYELLOW, EWGREEN, TMRESET;
  reg [2:0] Sreg, Snext;       // State register and next state
  parameter NSGO    = 3'b000,  // State encodings
            NSWAIT  = 3'b001,
            NSWAIT2 = 3'b010,
            NSDELAY = 3'b011,
            EWGO    = 3'b100, 
            EWWAIT  = 3'b101,
            EWWAIT2 = 3'b110,
            EWDELAY = 3'b111;

  always @ (posedge CLOCK)	// Create state memory with sync reset
    if (RESET) Sreg <= NSDELAY; else Sreg <= Snext;

  always @ (NSCAR or EWCAR or TMSHORT or TMLONG or Sreg) // Next-state logic.
    case (Sreg)
      NSGO :                                      // North-south green.
        if      (~TMSHORT)        Snext = NSGO;   // Minimum 5 seconds.
        else if (TMLONG)          Snext = NSWAIT; // Maximum 5 minutes.
        else if ( EWCAR & ~NSCAR) Snext = NSGO;   // Make EW car wait.
        else if ( EWCAR &  NSCAR) Snext = NSWAIT; // Thrash if cars both ways.
        else if (~EWCAR &  NSCAR) Snext = NSWAIT; // New NS car? Make it stop!
        else                      Snext = NSGO;   // No one coming, stay as is.
      NSWAIT  : Snext = NSWAIT2;                  // Yellow light,
      NSWAIT2 : Snext = NSDELAY;                  //   two ticks for safety.
      NSDELAY : Snext = EWGO;                     // Red both ways for safety.
      EWGO    :                                   // East-west green.
        if      (~TMSHORT)        Snext = EWGO;   // Same behavior as above.
        else if (TMLONG)          Snext = EWWAIT;             
        else if ( NSCAR & ~EWCAR) Snext = EWGO;   
        else if ( NSCAR &  EWCAR) Snext = EWWAIT; 
        else if (~NSCAR &  EWCAR) Snext = EWWAIT; 
        else                      Snext = EWGO;   
      EWWAIT  : Snext = EWWAIT2;                     
      EWWAIT2 : Snext = EWDELAY;                     
      EWDELAY : Snext = NSGO;                        
      default : Snext = NSDELAY;                  // "Reset" state.   
    endcase
                                                  // Output logic.
  assign TMRESET  = (Sreg==NSWAIT2 || Sreg==EWWAIT2) ? 1 : 0;
  assign NSRED    = (OVERRIDE) ?  FLASHCLK  :
                         (Sreg!=NSGO && Sreg!=NSWAIT && Sreg!=NSWAIT2);
  assign NSYELLOW = (OVERRIDE) ?  0  : (Sreg==NSWAIT || Sreg==NSWAIT2);
  assign NSGREEN  = (OVERRIDE) ?  0  : (Sreg==NSGO);
  assign EWRED    = (OVERRIDE) ?  FLASHCLK  :
                         (Sreg!=EWGO && Sreg!=EWWAIT && Sreg!=EWWAIT2);
  assign EWYELLOW = (OVERRIDE) ?  0  : (Sreg==EWWAIT || Sreg==EWWAIT2);
  assign EWGREEN  = (OVERRIDE) ?  0  : (Sreg==EWGO);
endmodule