module Vrsvaleoc ( CLOCK, RESET, NSCAR, EWCAR, TMSHORT, TMLONG,
                   OVERRIDE, FLASHCLK, NSRED, NSYELLOW, NSGREEN,
                   EWRED, EWYELLOW, EWGREEN, TMRESET );
  input CLOCK, RESET, NSCAR, EWCAR, TMSHORT, TMLONG, OVERRIDE, FLASHCLK;
  output NSRED, NSYELLOW, NSGREEN, EWRED, EWYELLOW, EWGREEN, TMRESET;
  reg [1:7] Sreg, Snext;       // State register and next state
  // bit positions of output-coded assignment: [1] NSRED, [2] NSYELLOW, [3] NSGREEN,
  //                                [4] EWRED, [5] EWYELLOW, [6] EWGREEN, [7] EXTRA
  parameter NSGO    = 7'b0011000,  // State encodings
            NSWAIT  = 7'b0101000,
            NSWAIT2 = 7'b0101001,
            NSDELAY = 7'b1001000,
            EWGO    = 7'b1000010, 
            EWWAIT  = 7'b1000100,
            EWWAIT2 = 7'b1000101,
            EWDELAY = 7'b1001001;

  always @ (posedge CLOCK)   // Create state memory with sync reset
    if (RESET) Sreg <= NSDELAY; else Sreg <= Snext;

  always @ (NSCAR or EWCAR or TMSHORT or TMLONG or Sreg) // Next-state logic.
    case (Sreg)
      NSGO :                                      // North-south green.
        if      (~TMSHORT)        Snext = NSGO;   // Minimum 5 seconds.
        else if (TMLONG)          Snext = NSWAIT; // Maximum 5 minutes.
        else if ( EWCAR & ~NSCAR) Snext = NSGO;   // Make EW car wait.
        else if ( EWCAR &  NSCAR) Snext = NSWAIT; // Thrash if cars both ways.
        else if (~EWCAR &  NSCAR) Snext = NSWAIT; // New NS car? Make it stop!
        else                      Snext = NSGO;   // No one coming, stay as is.
      NSWAIT  : Snext = NSWAIT2;                  // Yellow light,
      NSWAIT2 : Snext = NSDELAY;                  //   two ticks for safety.
      NSDELAY : Snext = EWGO;                     // Red both ways for safety.
      EWGO    :                                   // East-west green.
        if      (~TMSHORT)        Snext = EWGO;   // Same behavior as above.
        else if (TMLONG)          Snext = EWWAIT;             
        else if ( NSCAR & ~EWCAR) Snext = EWGO;   
        else if ( NSCAR &  EWCAR) Snext = EWWAIT; 
        else if (~NSCAR &  EWCAR) Snext = EWWAIT; 
        else                      Snext = EWGO;   
      EWWAIT  : Snext = EWWAIT2;                     
      EWWAIT2 : Snext = EWDELAY;                     
      EWDELAY : Snext = NSGO;                        
      default : Snext = NSDELAY;                  // "Reset" state.   
    endcase
                                                  // Output logic.
  assign TMRESET  = (Sreg==NSWAIT2 || Sreg==EWWAIT2);
  assign NSRED    = Sreg[1];
  assign NSYELLOW = Sreg[2];
  assign NSGREEN  = Sreg[3];
  assign EWRED    = Sreg[4];
  assign EWYELLOW = Sreg[5];
  assign EWGREEN  = Sreg[6];
endmodule